;************************************************************************
;*        	 SST FlashFlex51 Boot-Strap Loader(BSL)     		*
;*    		     (F51MBLE3.A51 - Version 1.1E) 			*
;*		 (for SST89C58 MCU - Device Code: E2h) 			*
;* 	 (MCU with 11.0592 MHz Ext. Crystal/38.4 Kbps baud rate)     	*
;*									*
;* NOTE:								*
;* 1. It's no longer need to check the port pin P1.0 to decide whether 	*
;*    to execute BSL IAP cmd or to run user code during BSL routine. 	*
;* 2. In BSL, the firmware(fw) sets the Watchdog Timer(WDT) for a time 	*
;*    period of 50 ms, sends a query cmd(F7h) to host PC and waits for 	*
;*    hand-shaking cmd(05h & 55h) from PC before WDT timeout. Upon 	*
;*    detecting the hand-shaking cmd, the fw sends its ID - three 	*
;*    consecutive bytes: device code, fw version and fw revision - to 	*
;*    host PC, and then waits for Pseudo IAP cmd from PC with new time-	*
;*    out of 2 seconds.                                                 *
;* 3. The fw will execute user code at 0000h after WDT timeout. 	*
;* 4. From the host PC, the user can issue a run-usercode cmd(62h & 62h)*
;*    to stop BSL mode and to executes user code instead.     	        *
;* 5. The fw supports a 38.4 Kbps baud for serial port with an external *
;*    OSC of 12 MHz. The previous version only supports a 9.6 Kbps baud.* 
;* 6. Unlike previous version, this fw uses a polling algorithm rather 	*
;*    than using interrupt-driven rs232_isr to access serial port. For 	*
;*    users plan to overwrite the fw in block 1 of flash, this feature 	*
;*    can facilitate the incorporation of user code into BSL routines.	*
;* 7. The pseudo IAP cmd of SECURITY-LOCK(SFCM_SB) and BLOCK-ERASE	*
;*    (SFCM_BE) has been moved to External Memory Mode(EA# = 1)- please *
;*    refers to F51EBLE.A51/F51EBLF.A51 of fw v1.1E.                    *
;* 8. This fw is referred as the "Internal Memory Mode(EA# = 1)" by the *
;*    host PC and is loaded into block 1 of flash. To execute the BSL 	*
;*    after reset, the user needs to remap 1KB/2KB/4KB of flash memory 	*
;*    from External Memory Mode or External Host Mode(RST = 1 and PSEN# *
;*    = 0) which often used by a standalone universal programmer.	* 
;* 9. FW withholds the following resources:                             *
;*    Registers R0-R6 in register bank 0; Internal RAM 08h-0Fh;         * 
;*    Watchdog Timer (WDT); Registers: A, B, DPTR, SFCF registers.      *
;*10. Companion SST89C54/C58 MCU datasheet/specifications should be	*
;*    reviewed in conjunction with this application note for a complete	*
;*    understand of the device.						*
;*									*
;************************************************************************
;* Developer: 	Frank Cirimele						*
;*		SST Embedded Controller Application Engineering		*
;*									*
;* Contact:	Silicon Storage Technology Inc.				*
;*		1171 Sonora Court					*
;*		Sunnyvale, CA 94086					*
;*		phone: 	(408)522-7352					*
;*		fax:	(408)749-9658					*
;*		email: 	cirimele@ssti.com				*
;*									*
;* Original Code Release Date: 6-08-2001			       	*
;*									*
;************************************************************************

;************************************************************************
;*                                                                      *
;* 8052 Family Special Function Registers(SFRs) Memory Locations	*
;*                                                                      *
;************************************************************************

T2CON 		DATA    0C8H	; Timer/Counter 2 Control
RCAP2L          DATA    0CAH    ; Auto-Reload/Capture 2 Low Byte
RCAP2H          DATA    0CBH    ; Auto-Reload/Capture 2 High Byte
TL2             DATA    0CCH    ; Timer/Counter 2 Low Byte
TH2             DATA    0CDH    ; Timer/Counter 2 High Byte

;************************************************************************
;*                                                                      *
;* 	FlashFlex51 MCU SST89C54/58 SFR Memory locations  		*
;*                                                                      *
;************************************************************************

SFCF		DATA    0B1H	; SuperFlash Configuration
SFCM		DATA    0B2H	; SuperFlash Command
SFAL		DATA    0B3H	; SuperFlash Address Low
SFAH		DATA    0B4H	; SuperFlash Address High
SFDT		DATA    0B5H	; SuperFlash Data
SFST		DATA    0B6H	; SuperFlash Status
WDTC		DATA    0C0H	; Watchdog Timer Control
WDTD		DATA     86H	; Watchdog Timer Data/Reload

;************************************************************************
;*                                                                      *
;* 	FlashFlex51 MCU SST89C54/58 IAP Commands 	  		*
;*                                                                      *
;************************************************************************

SFCM_SE		EQU	0BH	; Sector erase cmd
SFCM_PB		EQU	0EH	; Program byte cmd
SFCM_BP		EQU	06H	; Burst program cmd
SFCM_VB		EQU	0CH	; Verify byte cmd

;************************************************************************
;*                                                                      *
;* 		Variables Definition	 				*
;*                                                                      *
;************************************************************************

CMD		EQU	R2	; PC command
PCDATA  	EQU	R3	; PC data 
ADRHI   	EQU	R4	; start address - high byte
ADRLO 		EQU	R5	; start address - low byte
COUNT   	EQU	R6	; byte count or size

STR1            DATA    08h     ; internal RAM 08h - 0Bh is used to store STRG
STR2            DATA    09h
STR3            DATA    0Ah
STR4            DATA    0Bh

;************************************************************************
;*                                                                      *
;* 		Constants Definition	 				*
;*                                                                      *
;************************************************************************

SST89C54_ID	EQU     0E4h    ; Device code of SST89C54 MCU
SST89C58_ID	EQU     0E2h    ; Device code of SST89C58 MCU 
FW_Ver          EQU      11h    ; BSL firmware version 1.1
FW_Rev          EQU      'E'    ; BSL firmware revision E, ie. BSL v1.1E

;************************************************************************
;*                                                                      *
;* 	   RESET AND INTERRUPT VECTORS (SECTOR 0 in BLOCK 1 of FLASH)	*
;*                                                                      *
;************************************************************************
        org 0F000h       	; reset vector after remapped 1K/2K/4K
        ljmp start  	        ; jump to start code

        org 0F003h       	; external interrupt 0 vector

        org 0F00Bh       	; timer 0 overflow vector

        org 0F013h       	; external interrupt 1 vector

        org 0F01Bh       	; timer 1 overflow vector
        
        org 0F023h       	; serial I/O interrupt vector

	org 0F02Bh		; timer 2 overflow vector

;************************************************************************
;*									*									
;*		      	 BOOT CODE OR USER CODE?			*
;*									*								*
;************************************************************************
;* During BSL start routine, the fw check the WDTC.2 (Watchdog timer	*
;* reset flag) and jump to BSL poweron routine if WDTC.2 = 0 or compare *
;* STR1 thru STR4 with string "POWR" or "USER" if WDTC.2 = 1 (timeout 	*
;* detected).			*
;*   					    				*
;* After WDT timeout, the firmware takes the following steps:  		*
;* 1. Clear WDT reset flag if WDT expires in BSL1.1E, rather than in	*
;*    user code.							*
;* 2. Jump to resetval routine to restore SFRs reset values.		*
;* 3. Jump to 0000h in block 0 of flash to execute user code.         	*
;*									*								*
;************************************************************************
	org  0F040h-2
        sjmp $

	org  0F040h        

start:	mov  a, SFCF            ; jump to chkwdtc if remap is set
	anl  a, #00000011b
	jnz  chkwdtc
	
	mov  STR1, #"R"         ; if remap is not set, then set flag of "RMAP"
	mov  STR2, #"M"
	mov  STR3, #"A"
	mov  STR4, #"P"

	mov  WDTD, #-2          ; wait 10ms(typ.) to reset MCU
	mov  WDTC, #0fh         ; in order to detect real remapping value
	sjmp $                  ; software trap remapping

chkwdtc:jnb  WDTC.2,  poweron   ; jump to init after Power-On-Reset

        mov  a, STR1
        cjne a, #'P', others
        mov  a, STR2
        cjne a, #'O', others
        mov  a, STR3
        cjne a, #'W', others
        mov  a, STR4
        cjne a, #'R', others

	orl  WDTC, #00000100b   ; clear the reset flag ( WDTS, ie WDTC.2 )
	ljmp resetval           ; long jump to resetval if flag is "POWR"

others: mov  a, STR1
        cjne a, #'U', rmap
        mov  a, STR2
        cjne a, #'S', rmap
        mov  a, STR3
        cjne a, #'E', rmap
        mov  a, STR4
        cjne a, #'R', rmap
                                ; jump to resetval if flag="USER" and WDTC.2=1
	ljmp resetval           ; but don't clear WDT reset flag (WDTS) !!

rmap:	mov  a, STR1
        cjne a, #'R', user
        mov  a, STR2
        cjne a, #'M', user
        mov  a, STR3
        cjne a, #'A', user
        mov  a, STR4
        cjne a, #'P', user
	orl  WDTC, #00000100b   ; clear reset flag
	sjmp poweron

user:	mov  STR1,   #'U'       ; set flag of "USER"
	mov  STR2,   #'S'
	mov  STR3,   #'E'
	mov  STR4,   #'R'
	sjmp init

;************************************************************************
;*									*
;*     		BSL INIT ROUTINE:   INITIALIZATION CODE	 		*
;*									*
;************************************************************************
;* 1. Initialize SFRs, set up serial connection and start WDT. 		*
;* 2. MCU sends Query cmd(F7h) to host PC and waits for hand-shaking	*
;*    cmd(05h & 55h) from PC and then reports the status byte to PC.	*
;* 3. MCU sends cmd F7h to PC and waits for cmd 60h from PC and	report	*
;*    the chip ID and fw ID to PC. 					*
;* 4. The fw stays in a loop routine: MCU sends cmd F7h to PC and waits *
;*    for pseudo IAP cmd from PC.					*
;* 5. To stop the loop routine, either apply a reset or the PC sends a 	*
;*    Run-usercode cmd(62h) to MCU.			       		*
;*									*
;************************************************************************
poweron:                        ; set flag of "POWR"
	mov  STR1,   #'P'
	mov  STR2,   #'O'
	mov  STR3,   #'W'
	mov  STR4,   #'R'

init:	clr  a                  ; clear Accumulator
	mov  PSW, a     	; clear Program Status Word
        mov  IE, a              ; clear Interrupt Enable
	mov  SP,     #0Bh       ; 0Ch thru 0Fh used as Stack!
	anl  SFCF,   #00111111b ; VIS=0, IAPEN=0
	mov  SFDT,   #0         ; any value other than 55h
	mov  SFAH,   #0abh      ; point to off-chip memory
	mov  SFAL,   #0cdh
        mov  RCAP2H, #high(-9)  ; 11.0592 MHz OSC, for 38.4K bps
        mov  RCAP2L, #low(-9)   ; 11.0592 MHz OSC, for 38.4K bps

;	mov  RCAP2H, #high(-10) ; 12 MHz OSC, for 38.4K bps
;	mov  RCAP2L, #low(-10)  ; 12 MHz OSC, for 38.4K bps

       	mov  T2CON,  #00110100b	; Timer2 as baudrate generator for RCLK+TCLK  

        mov  SCON,   #01010010b ; mode1, 8bit UART, no parity, REN=1, TI=1
        mov  WDTD,   #-5	; count up to 5 x 7.7ms=38ms(min.), 50ms(typ.)
	mov  WDTC,   #00001111b ; enable reset, clear flag, refresh & start WDT

;************************************************************************
;*									*
;*			LOOP ROUTINE                                    *
;*									*
;************************************************************************
;* MCU sends a Query cmd(F7h) to notify the host PC that it's ready to 	*
;* receive pseudo IAP command and then waits indefinitely for commands.	*
;*									*
;************************************************************************
loop:   mov   a, #0F7h          ; store Query cmd(0F7H ) at ACC 
	jnb   TI, $             ; wait until end of last byte transmitting
	mov   SBUF, a           ; send Query cmd to serial port
	clr   TI                ; busy on transmitting...

       	mov   B,    #0          ; invalid IAP command

waitpc:	jnb   RI, waitpc        ; wait until incoming byte is ready
	mov   a,  SBUF          ; transfer the received byte into ACC
	clr   RI                ; ready to get next byte

	mov   CMD, a            ; copy ACC into CMD
	setb  WDTC.1            ; refresh watchdog timer!

;************************************************************************
;*									*
;*	  	HAND-SHAKING ROUTINE (Pseudo IAP CMD: 05h & 55h)	*
;*									*
;************************************************************************
;* 1. Check the hand-shaking cmd sequence(05h & 55h) and report the	*
;*    status byte to host PC. 						*
;* 2. Continue the BSL loop routine if invalid cmd detected.		*
;*									*
;************************************************************************

	cjne  a, #05h, cmd60	; jump to cmd60 if cmd != 05h
        setb  WDTC.1            ; refresh watchdog timer!
	lcall INBYTE            ; read a byte from serial port
	cjne  a, #55h, loop     ; continue the loop routine if invalid 

	mov   a, SFCF		; read SFCF to check status of MAP_EN(bit 1,0)
	anl   a, #00000011b	; combine SECD(bit 7,6,5) & MAP_EN(bit 1,0)
	orl   a, SFST    	; into one byte and send to host PC
	lcall OUTBYTE		; send the status byte to serial port	

	clr   WDTC.0            ; stop WDT for valid cmd
        mov   WDTD,  #-200	; change to 2 seconds for Watchdog Timer
	mov   WDTC,  #00001111b ; enable reset, clear flag, refresh & start WDT
	sjmp  loop              ; continue the BSL loop routine

;************************************************************************
;*									*
;*    	 REPORT CHIP-ID & FW-ID ROUTINE (Pseudo IAP CMD: 60h) 		*
;*									*
;************************************************************************
;* Host pc ask the MCU to report Chip ID and Fw ID:			*
;* 1. Send the Device code: E2h for SST89C58 or E4h for SST89C54	*
;* 2. Send the fw version ID: 11h for fw v1.1 				*
;* 3. Send the fw revision ID: 'E'					*  
;*									*
;************************************************************************
 
cmd60:	cjne  a, #60h, cmd62   
  	setb  WDTC.1            ; refresh watchdog timer!
	mov   a, #SST89C58_ID	; Device ID of SST89C58: E2h 
	lcall OUTBYTE           ; send Device ID to PC
	mov   a, #FW_Ver	
        lcall OUTBYTE	        ; send fw version to PC
	mov   a, #FW_Rev	
        lcall OUTBYTE           ; send fw revision to PC
	sjmp  loop              ; jump to loop routine

;************************************************************************
;*									*
;*    	 RUN USERCODE ROUTINE (Pseudo IAP CMD: 62h & 62h) 		*
;*									*
;************************************************************************
;* Host PC sends this cmd to Run User Code:                             *
;* 1. MCU must receive two consecutive bytes: 62h & 62h.		*
;* 2. Jump to resetval routine to restore SFRs reset values and	to 	*
;*    execute user code at 0000h in block 0 of flash.			*  
;*									*
;************************************************************************
 
cmd62:  cjne  a, #62h, nop_cmd  ; jump to nop_cmd routine if not 62h
	lcall INBYTE            ; read one more byte from serial port
        cjne  a, #62h, loop     ; Invalid cmd sequence if 2nd byte != 62h

	jnb   TI,$              ; wait until last byte transmitted over
        ljmp  resetval          ; jump to resetval routine & run usercode

;************************************************************************
;*									*
;*    	      BRANCH TO OTHER PSEUDO IAP CMD ROUTINES  			*
;*									*
;************************************************************************

nop_cmd:cjne  a, #61h, cmd2 
	setb  WDTC.1            ; refresh watchdog timer!
	ljmp  loop              ; CMD 61H = NOP cmd

cmd2:	cjne  a, #SFCM_SE, cmd3
	lcall byte3
	mov   SFDT, #55h
	lcall sector_e          ; sector erase
	ljmp  loop

cmd3:	cjne  a, #SFCM_PB, cmd4
	lcall byte3
 	lcall program_b         ; program byte
	ljmp  loop  

cmd4:  	cjne  a, #SFCM_BP, cmd5
	lcall byte3
	lcall burst_p           ; burst program
	ljmp  loop

cmd5: 	cjne  a, #SFCM_VB, wrongcmd
	lcall byte3
	lcall verify_b          ; verify byte
	ljmp  loop

wrongcmd:                 
        ljmp loop		; continue the BSL loop routine
       	nop
        nop
        nop
        sjmp $		      	; software trap
;************************************************************************
byte3:  lcall INBYTE            ; wait 3 bytes from PC
	mov   ADRHI, a          ;  1) input high address byte
	mov   DPH, a
      
        lcall INBYTE		;  2) input low address byte
        mov   ADRLO, a
	mov   DPL, a
     
        lcall INBYTE		;  3) input count
        mov   COUNT, a

	mov   r0,  #0           ; clear r0 with no timeout
        ret
;************************************************************************
       	nop
        nop
        nop
        sjmp $		      	; software trap

;************************************************************************
;*									*
;*   SERIAL COMMUNICATION SUBROUTINE (Polling Algorithm)		*
;*									*
;************************************************************************

OUTBYTE:                        ; Transmit a byte thru serial port
	jnb   TI, $             ; wait until end of last byte transmitting
	mov   SBUF, a           ; store the data byte at SBUF
	clr   TI                ; busy on transmitting...
	setb  WDTC.1            ; refresh watchdog timer!
	ret
;========================================================================
       	nop
        nop
        nop
        sjmp  $		      	; software trap
;========================================================================
INBYTE:	                        ; Receive a byte from serial port
	jnb   RI,$              ; wait untill incoming byte is ready
	mov   a, SBUF           ; transfer the received byte into ACC
	clr   RI                ; ready to get next byte

	setb  WDTC.1            ; refresh watchdog timer!
	ret
;========================================================================
       	nop
        nop
        nop
        sjmp  $		      	; software trap
;========================================================================

;************************************************************************ 
;                                                                       *
; Name:	SECTOR ERASE ROUTINE (IAP CMD: 0Bh)                             *
;                                                                       *
;	ENTRY:   (1) Starting Address (on a Sector Boundary)            *
;			R4 = Address Byte High                          *
;			R5 = Address Byte Low                           *
;		 (2) Sector Count (in R6)                               *
;                                                                       *
;	RETURNS:	ACC                                             *
;		                                                        *
;	DESTROYED:	ACC,B,R0,R1,R2,R3,R4,R5,R6                      *
;                                                                       *
; This routine erases the sector(s) requested, beginning from the       *
; starting address provided.  Notification is returned to the PC indi-	* 
; cating success (ACC.2 = 0) or failure (ACC.2 = 1) of this operation.  *
;                                                                       *
;************************************************************************ 
			
sector_e:
	mov   B, #SFCM_SE
	mov   a, CMD
 	cjne  a, #SFCM_SE, error1
	mov   a, SFDT
	cjne  a, #55h, error2

nxtsctr:mov   a, ADRHI
	clr   c
	subb  a, #0f0h
	jnc   error3
 
	orl   SFCF, #0C0h       ; set VIS & IAPEN bits = 1
	mov   SFAH, ADRHI	; load high order address byte
	mov   SFAL, ADRLO	; load low order address byte
	mov   SFCM, B		; issue sector erase command
        mov   CMD,  #04h        ; mask Flash_Busy bit in done?
	lcall done?		; wait for the sector erase operation
	jb    acc.2, exit2	; if timeout err detected, terminate
	djnz  COUNT, newaddr

exit2:	lcall OUTBYTE
	anl   SFCF, #00111111b  ; VIS=IAPEN=0
	mov   SFDT, #0          ; any value other than 55h
	mov   SFAH, #0abh       ; point to off-chip memory
	mov   SFAL, #0cdh
	mov   B,    #0          ; invalid IAP command
	ret			

error1: mov   a,    #0C1h       ; cmd is not valid Sector-Erase (0Bh)
	sjmp  exit2

error2: mov   a,    #0C2h       ; SFDT is not valid 55h ( double-guard )
	sjmp  exit2

error3: mov   a,    #0C3h       ; sector address points to Block1
	sjmp  exit2

newaddr:mov   a, ADRLO
	add   a, #128           ; sector size of Block0 is 128
	mov   ADRLO, a		; low-byte address updated for next sector
	mov   a, ADRHI
	addc  a, #0
	mov   ADRHI, a		; high-byte address updated for next sector
        sjmp  nxtsctr
;========================================================================
       	nop
        nop
        nop
        sjmp  $ 	      	; software trap
;========================================================================

;************************************************************************ 
;                                                                       *
; Name:	PROGRAM BYTE ROUTINE (IAP CMD: 0Eh)                             *
;                                                                       *
;	INPUTS: (1) Starting Address:                                   *
;		    DPH = Address Byte High                             *
;		    DPL = Address Byte Low                              *
;		(2) Byte Count (in R6)                                  *
;                                                                       *
;	RETURNS: NONE          						*
;                                                                       *
;	DESTROYED:	ACC,B,R0,R1,R2,R3,R6,DPTR                        *
;		                                                        *
; This routine receives the Starting Address and Length of Bytes to     *
; Write from the main loop.  It then writes the data bytes received     *
; into selected allocation byte.                                        *
;                                                                       *
;************************************************************************ 
	
program_b:
	mov   B, #SFCM_PB
 	mov   a, DPH
	clr   c
	subb  a, #0f0h
	jnc   exit3 

	lcall INBYTE            ; fetch a byte from serial port
	orl   SFCF, #0C0h       ; set VIS & IAPEN bits = 1
        mov   SFAH, DPH		; set address to write
        mov   SFAL, DPL
        mov   SFDT, A     	; set data 
	mov   SFCM, B		; issue program-byte cmd
        mov   CMD,  #04h        ; mask Flash_Busy bit in done?
        lcall done?		; wait for command to complete

	anl   SFCF, #00111111b  ; VIS=IAPEN=0
        inc   DPTR       	; point to next byte address to be written
	djnz  COUNT, program_b	; repeat loop until all data written

exit3:	anl   SFCF, #00111111b  ; VIS=IAPEN=0
	mov   SFDT, #0          ; any value other than 55h
	mov   SFAH, #0abh       ; point to off-chip memory
	mov   SFAL, #0cdh
	mov   B,    #0          ; invalid IAP command
        ret	
;========================================================================
       	nop
        nop
        nop
        sjmp  $		      	; software trap
;========================================================================

;************************************************************************ 
;                                                                       *
; Name:	BURST PROGRAM ROUTINE (IAP CMD: 06h)                            *
;                                                                       *
;	INPUTS: (1) Starting Address:                                   *
;		    DPH = Address Byte High                             *
;		    DPL = Address Byte Low                              *
;		(2) Byte Count (in R6)                                  *
;                                                                       *
;	RETURNS: final state of flash operation in ACC, also send to PC *
;                                                                       *
;	DESTROYED: ACC,B,R0,R1,R2,R3,R6,DPTR                            *
;                                                                       *
; This routine is similar to above program_b, it receives Starting Addr.* 
; and Length of Bytes to Write from the main loop, and then writes the  *
; bytes got from PC into selected allocation byte with Burst-Program    * 
; mode.                                                                 *
;************************************************************************ 

burst_p:
	mov   B, #SFCM_BP
 	mov   a, DPH            ; can NOT burst-program Block1
	clr   c
	subb  a, #0f0h
	jnc   exit4 

	lcall INBYTE            ; fetch a byte from serial port
        orl   SFCF, #0C0h       ; set VIS & IAPEN bits = 1
        mov   SFAH, DPH		; set address to write
        mov   SFAL, DPL
        mov   SFDT, A      	; set data 
        mov   SFCM, B		; issue burst-program cmd
        mov   CMD,  #08h        ; mask Busy bit in done? (Not Flash_busy bit!)
        lcall done?      	; wait for burst-program to complete
	anl   SFCF, #00111111b  ; VIS=IAPEN=0
        inc   DPTR       	; point to next byte address to be written
	djnz  COUNT, burst_p	; repeat until all data written

	mov   CMD,  #04h        ; mask SFST.2 to check Flash_Busy bit
	lcall done?		; wait for flash operation to finish
	lcall OUTBYTE           ; send status to PC

exit4: 	anl   SFCF, #00111111b  ; VIS=IAPEN=0
	mov   SFDT, #0          ; any value other than 55h
	mov   SFAH, #0abh       ; point to off-chip memory
	mov   SFAL, #0cdh
	mov   B,    #0          ; invalid IAP command
        ret	
;========================================================================
       	nop
        nop
        nop
        sjmp  $		      	; software trap
;========================================================================

;************************************************************************ 
;                                                                       *
; Name:	VERIFY BYTE ROUTINE (IAP CMD: 0Ch)                              *
;                                                                       *
;	INPUTS:	 (1) Starting Address:                                  *
;		     DPH = Address Byte High                            *
;		     DPL = Address Byte Low                             *
;		 (2) Sector Byte Length (in R6)                         *
;                                                                       *
;	RETURNS: ACC and SFDT contains the last byte data to be read    *
;                                                                       *
;	DESTROYED: ACC,DPTR,R6                                          *
;		                                                        *
; This routine reads the data bytes beginning at the flash memory loca-	*
; tion requested, and sends each data byte read immediately to the PC.  *
;                                                                       *
;************************************************************************ 

verify_b:
 	mov   a, DPH            ; can NOT burst-program Block1
	clr   c
	subb  a, #0f0h
	jc    block0 

	orl   SFCF, #80h        ; VIS=1
	clr   a
	movc  a,  @a+dptr       ; use movc to read contents in block1
	sjmp  tellpc

block0: orl   SFCF, #0C0h       ; set VIS & IAPEN bits = 1
 	mov   SFAH, DPH		; load sfah with high address byte
	mov   SFAL, DPL		; load sfal with low address byte
        mov   SFCM, #SFCM_VB	; issue verify byte command
	nop
	nop
	nop
	mov   a, SFDT			
tellpc:	lcall OUTBYTE           ; send data byte to PC

	inc   DPTR		; point to next data byte address
	djnz  COUNT, verify_b	; read next byte until count reaches zero

 	anl   SFCF, #00111111b  ; VIS=IAPEN=0
	mov   SFDT, #0          ; any value other than 55h
	mov   SFAH, #0abh       ; point to off-chip memory
	mov   SFAL, #0cdh
	mov   B,    #0          ; invalid IAP command
	ret			
;========================================================================
       	nop
        nop
        nop
        sjmp  $		      	; software trap
;========================================================================
			
;************************************************************************
;*									*
;*	     CHECK WHETHER THE FLASH OPERATION IS COMPLETED	 	*
;*									*
;************************************************************************
;                                                                       *
; INPUT:   CMD = 04h to detect SFST.2 ( Flash_Busy ) or                 *
;              = 08h to detect SFST.3 ( Busy )                          *
;                                                                       *
; RETURN:  flag ACC.2=1 If Flash operation is still busy after elapse 	*
; of maximum 3000 cycles (about 3.25ms at 11.0592 MHz cystal).          *
; otherwise, flag ACC.2=0 means successful operation                    *
;                                                                       *
;************************************************************************

done?:  mov  r1, #250      	; 250 * (1+1+6+2+2) = 3000 cycles
busy:   mov  a,  SFST           ; 1 cycle
	anl  a,  CMD		; 1 cycle
	nop			; 6 cycles of nop
	nop
	nop
	nop
	nop
	nop
	jz   ready              ; 2 cycles
 	djnz r1, busy           ; 2 cycles

	mov  a,  #11000100b    	; ACC.2=1 indicates flash timeout error
  	sjmp status

ready:  mov  a,  #0C0h          ; ACC.2=0 means successful operation
status:	orl  a,  r0             ; r0 keeps history of timeout
	mov  r0, a
        ret
;========================================================================
       	nop
        nop
        nop
        sjmp  $		      	; software trap
;========================================================================
;************************************************************************
;*									*
;*	     RESETVAL ROUTINE 					 	*
;*                                                                      *
;************************************************************************
;* 1. Turn on / off WDT based on Remapping range.                       *
;* 2. Restore Reset values into SFRs.					*
;* 3. Execute User's Code starting at 0000h in block 0.	      		*
;*                                                                      *
;************************************************************************

resetval:
	mov  a, SFCF
	anl  a,    #00000011b   ; MASK MAP_EN1 & MAP_EN0
	anl  WDTC, #00000100b   ; clear all other bits except WDTC.2
	cjne a, #3, register
	orl  WDTC, #00001011b   ; turn on WDT before exits from BSL1.1E

register: 
	CLR  A
        MOV  B,    A
	MOV  PSW,  A
        MOV  SP,   #07H
        MOV  DPTR, #0
        MOV  IE,   #40H
        MOV  IP,   A               
        MOV  PCON, A
        ANL  SFST, #11100000B
        MOV  SFCM, A
        MOV  SFDT, A
        MOV  SFAL, A
        MOV  SFAH, A
        MOV  RCAP2L, A
        MOV  RCAP2H, A
        MOV  TL2,  A
        MOV  TH2,  A
        MOV  T2CON,A
        MOV  SCON, A
        MOV  TCON, A
        MOV  TMOD, A
        MOV  TL0,  A
        MOV  TH0,  A
        MOV  TL1,  A
        MOV  TH1,  A
	mov  STR1, #'D'     ; destroy data at 08h,09h,0Ah,0Bh in SRAM
	mov  STR2, #'O'   
	mov  STR3, #'N'
	mov  STR4, #'E'
	orl  SFCF, #80h     ; VIS=1 !!
	anl  SFCF, #80h     ; VIS=1, IAPEN=0, REMAP=0KB
	ljmp 0000h          ; jump to user's code at 0000h in block 0 
;=====================================================================
       	nop
        nop
        nop
	sjmp $
        END


